@@ -4,16 +4,22 @@
 # LICENSE file in the root directory of this source tree.
 
 # Calculate R-squared
-get_rsq <- function(true, predicted, p = NULL, df.int = NULL) {
+get_rsq <- function(true, predicted, p = NULL, df.int = NULL, n_train = NULL) {
   sse <- sum((predicted - true)^2)
   sst <- sum((true - mean(true))^2)
-  rsq <- 1 - sse / sst
-  if (!is.null(p) & !is.null(df.int)) {
-    n <- length(true)
+  rsq <- 1 - sse / sst # rsq interpreted as variance explained
+  rsq_out <- rsq
+  if (!is.null(p) && !is.null(df.int)) {
+    if (!is.null(n_train)) {
+      n <- n_train # for oos dataset, use n from train set for adj. rsq
+    } else {
+      n <- length(true)
+    }
     rdf <- n - p - 1
-    rsq <- 1 - (1 - rsq) * ((n - df.int) / rdf)
+    rsq_adj <- 1 - (1 - rsq) * ((n - df.int) / rdf)
+    rsq_out <- rsq_adj
   }
-  return(rsq)
+  return(rsq_out)
 }
 
 # Robyn colors
@@ -36,9 +42,9 @@ flatten_hyps <- function(x) {
   if (is.null(x)) {
     return(x)
   }
-  temp <- sapply(x, function(x) {
+  temp <- unlist(lapply(x, function(x) {
     sprintf("[%s]", paste(if (is.numeric(x)) signif(x, 6) else x, collapse = ", "))
-  })
+  }))
   paste(paste0("  ", names(temp), ":"), temp, collapse = "\n")
 }
 
@@ -47,7 +53,9 @@ flatten_hyps <- function(x) {
 #'
 #' Update Robyn version from
 #' \href{https://github.com/facebookexperimental/Robyn}{Github repository}
-#' for "dev" version or from CRAN (not yet submitted, but soon!).
+#' for latest "dev" version or from
+#' \href{https://CRAN.R-project.org/package=Robyn}{CRAN}
+#' for latest "stable" version.
 #'
 #' @param dev Boolean. Dev version? If not, CRAN version.
 #' @param ... Parameters to pass to \code{remotes::install_github}
