@@ -39,8 +39,9 @@
 #' @export
 robyn_clusters <- function(input, dep_var_type, all_media = NULL, k = "auto", limit = 1,
                            weights = rep(1, 3), dim_red = "PCA",
-                           quiet = FALSE, export = FALSE,
+                           quiet = FALSE, export = FALSE, seed = 123,
                            ...) {
+  set.seed(seed)
   if ("robyn_outputs" %in% class(input)) {
     if (is.null(all_media)) {
       aux <- colnames(input$mediaVecCollect)
@@ -64,7 +65,7 @@ robyn_clusters <- function(input, dep_var_type, all_media = NULL, k = "auto", li
     }
   }
 
-  ignore <- c("solID", "mape", "decomp.rssd", "nrmse", "pareto")
+  ignore <- c("solID", "mape", "decomp.rssd", "nrmse", "nrmse_test", "nrmse_train", "pareto")
 
   # Auto K selected by less than 5% WSS variance (convergence)
   min_clusters <- 3
@@ -72,7 +73,10 @@ robyn_clusters <- function(input, dep_var_type, all_media = NULL, k = "auto", li
   if ("auto" %in% k) {
     cls <- tryCatch(
       {
-        clusterKmeans(df, k = NULL, limit = limit_clusters, ignore = ignore, dim_red = dim_red, quiet = TRUE) # , ...)
+        clusterKmeans(df,
+          k = NULL, limit = limit_clusters, ignore = ignore,
+          dim_red = dim_red, quiet = TRUE, seed = seed
+        ) # , ...)
       },
       error = function(err) {
         message(paste("Couldn't automatically create clusters:", err))
@@ -104,7 +108,8 @@ robyn_clusters <- function(input, dep_var_type, all_media = NULL, k = "auto", li
 
   # Select top models by minimum (weighted) distance to zero
   all_paid <- setdiff(names(cls$df), c(ignore, "cluster"))
-  top_sols <- .clusters_df(cls$df, all_paid, weights, limit)
+  ts_validation <- ifelse(all(is.na(cls$df$nrmse_test)), FALSE, TRUE)
+  top_sols <- .clusters_df(df = cls$df, all_paid, balance = weights, limit, ts_validation)
 
   # Build in-cluster CI with bootstrap
   ci_list <- confidence_calcs(xDecompAgg, cls, all_paid, dep_var_type, k, ...)
@@ -233,6 +238,8 @@ confidence_calcs <- function(xDecompAgg, cls, all_paid, dep_var_type, k, boot_n
       boot_ci = sprintf("[%s, %s]", round(.data$ci_low, 2), round(.data$ci_up, 2)),
       ci_low = .data$ci_low,
       ci_up = .data$ci_up,
+      sd = boot_se * sqrt(.data$n - 1),
+      dist100 = (.data$ci_up - .data$ci_low + 2 * boot_se * sqrt(.data$n - 1)) / 99,
       .groups = "drop"
     ) %>%
     ungroup()
@@ -244,13 +251,14 @@ confidence_calcs <- function(xDecompAgg, cls, all_paid, dep_var_type, k, boot_n
   ))
 }
 
-errors_scores <- function(df, balance = rep(1, 3)) {
+errors_scores <- function(df, balance = rep(1, 3), ts_validation = TRUE, ...) {
   stopifnot(length(balance) == 3)
-  error_cols <- c("nrmse", "decomp.rssd", "mape")
+  error_cols <- c(ifelse(ts_validation, "nrmse_test", "nrmse_train"), "decomp.rssd", "mape")
   stopifnot(all(error_cols %in% colnames(df)))
   balance <- balance / sum(balance)
   scores <- df %>%
     select(all_of(error_cols)) %>%
+    rename("nrmse" = 1) %>%
     mutate(
       nrmse = ifelse(is.infinite(.data$nrmse), max(is.finite(.data$nrmse)), .data$nrmse),
       decomp.rssd = ifelse(is.infinite(.data$decomp.rssd), max(is.finite(.data$decomp.rssd)), .data$decomp.rssd),
@@ -270,7 +278,7 @@ errors_scores <- function(df, balance = rep(1, 3)) {
       mape_w = balance[3] * .data$mape_n
     ) %>%
     # Calculate error score
-    mutate(error_score = (.data$nrmse_w^2 + .data$decomp.rssd_w^2 + .data$mape_w^2)^-(1 / 2)) %>%
+    mutate(error_score = sqrt(.data$nrmse_w^2 + .data$decomp.rssd_w^2 + .data$mape_w^2)) %>%
     pull(.data$error_score)
   return(scores)
 }
@@ -290,7 +298,10 @@ errors_scores <- function(df, balance = rep(1, 3)) {
 
   outcome <- removenacols(outcome, all = FALSE)
   outcome <- select(outcome, any_of(c("solID", all_media)))
-  errors <- distinct(x, .data$solID, .data$nrmse, .data$decomp.rssd, .data$mape)
+  errors <- distinct(
+    x, .data$solID, .data$nrmse, .data$nrmse_test,
+    .data$nrmse_train, .data$decomp.rssd, .data$mape
+  )
   outcome <- left_join(outcome, errors, "solID") %>% ungroup()
   return(outcome)
 }
@@ -305,12 +316,12 @@ errors_scores <- function(df, balance = rep(1, 3)) {
   (max - min) * (x - a) / (b - a) + min
 }
 
-.clusters_df <- function(df, all_paid, balance = rep(1, 3), limit = 1) {
+.clusters_df <- function(df, all_paid, balance = rep(1, 3), limit = 1, ts_validation = TRUE, ...) {
   df %>%
-    mutate(error_score = errors_scores(., balance)) %>%
+    mutate(error_score = errors_scores(., balance, ts_validation = ts_validation, ...)) %>%
     replace(., is.na(.), 0) %>%
     group_by(.data$cluster) %>%
-    arrange(.data$cluster, .data$solID, desc(.data$error_score)) %>%
+    arrange(.data$cluster, .data$error_score) %>%
     slice(1:limit) %>%
     mutate(rank = row_number()) %>%
     select(.data$cluster, .data$rank, everything())
