@@ -11,9 +11,11 @@
 #'
 #' @inheritParams robyn_outputs
 #' @param InputCollect \code{robyn_inputs()} output.
+#' @param OutputModels \code{robyn_run()} output.
 #' @param select_model Character. Which model ID do you want to export
 #' into the JSON file?
 #' @param dir Character. Existing directory to export JSON file to.
+#' @param all_sol_json Dataframe. Add all pareto solutions to json.
 #' @param ... Additional parameters.
 #' @examples
 #' \dontrun{
@@ -31,8 +33,11 @@ robyn_write <- function(InputCollect,
                         OutputCollect = NULL,
                         select_model = NULL,
                         dir = OutputCollect$plot_folder,
+                        OutputModels = NULL,
                         export = TRUE,
-                        quiet = FALSE, ...) {
+                        quiet = FALSE,
+                        all_sol_json = NULL,
+                        ...) {
   # Checks
   stopifnot(inherits(InputCollect, "robyn_inputs"))
   if (!is.null(OutputCollect)) {
@@ -48,7 +53,19 @@ robyn_write <- function(InputCollect,
   ret <- list()
   skip <- which(unlist(lapply(InputCollect, function(x) is.list(x) | is.null(x))))
   skip <- skip[!names(skip) %in% c("calibration_input", "hyperparameters", "custom_params")]
-  ret[["InputCollect"]] <- inputs <- InputCollect[-skip]
+  ret[["InputCollect"]] <- InputCollect[-skip]
+  if (is.null(OutputModels)) {
+    OutputModels <- OutputCollect$OutputModels
+  }
+  conv_msg <- mapply(function(x) x[[1]],
+    x = gregexpr(":", OutputModels$convergence$conv_msg),
+    SIMPLIFY = FALSE
+  )
+  conv_msg <- mapply(function(x, y) substr(x, 1, y - 1),
+    x = OutputModels$convergence$conv_msg,
+    y = conv_msg, USE.NAMES = FALSE
+  )
+  ret[["OutputCollect"]][["conv_msg"]] <- conv_msg
   # toJSON(inputs, pretty = TRUE)
 
   # ExportedModel JSON
@@ -71,7 +88,7 @@ robyn_write <- function(InputCollect,
       select(starts_with("rsq_"), starts_with("nrmse"), .data$decomp.rssd, .data$mape)
     outputs$hyper_values <- OutputCollect$resultHypParam %>%
       filter(.data$solID == select_model) %>%
-      select(contains(hyps_name), dplyr::ends_with("_penalty"), any_of(other_hyps)) %>%
+      select(contains(HYPS_NAMES), dplyr::ends_with("_penalty"), any_of(HYPS_OTHERS)) %>%
       select(order(colnames(.))) %>%
       as.list()
     outputs$hyper_updated <- OutputCollect$hyper_updated
@@ -84,13 +101,21 @@ robyn_write <- function(InputCollect,
     select_model <- "inputs"
   }
 
-  if (!dir.exists(dir)) dir.create(dir, recursive = TRUE)
+  if (!dir.exists(dir) & export) dir.create(dir, recursive = TRUE)
   filename <- sprintf("%s/RobynModel-%s.json", dir, select_model)
   filename <- gsub("//", "/", filename)
   class(ret) <- c("robyn_write", class(ret))
   attr(ret, "json_file") <- filename
   if (export) {
     if (!quiet) message(sprintf(">> Exported model %s as %s", select_model, filename))
+    if (!is.null(all_sol_json)) {
+      all_c <- unique(all_sol_json$cluster)
+      all_sol_json <- lapply(all_c, function(x) {
+        (all_sol_json %>% filter(.data$cluster == x))$solID
+      })
+      names(all_sol_json) <- paste0("cluster", all_c)
+      ret[["OutputCollect"]][["all_sols"]] <- all_sol_json
+    }
     write_json(ret, filename, pretty = TRUE)
   }
   return(invisible(ret))
@@ -136,6 +161,7 @@ print.robyn_write <- function(x, ...) {
     select(-contains("boot"), -contains("ci_")) %>%
     dplyr::rename_at("performance", list(~ ifelse(x$InputCollect$dep_var_type == "revenue", "ROI", "CPA"))) %>%
     mutate(decompPer = formatNum(100 * .data$decompPer, pos = "%")) %>%
+    dplyr::mutate_if(is.numeric, function(x) ifelse(!is.infinite(x), x, 0)) %>%
     dplyr::mutate_if(is.numeric, function(x) formatNum(x, 4, abbr = TRUE)) %>%
     replace(., . == "NA", "-") %>% as.data.frame())
 
@@ -144,10 +170,10 @@ print.robyn_write <- function(x, ...) {
   ))
 
   # Nice and tidy table format for hyper-parameters
-  hyps_name <- c(hyps_name, "penalty")
-  regex <- paste(paste0("_", hyps_name), collapse = "|")
+  HYPS_NAMES <- c(HYPS_NAMES, "penalty")
+  regex <- paste(paste0("_", HYPS_NAMES), collapse = "|")
   hyper_df <- as.data.frame(x$ExportedModel$hyper_values) %>%
-    select(-contains("lambda"), -any_of(other_hyps)) %>%
+    select(-contains("lambda"), -any_of(HYPS_OTHERS)) %>%
     tidyr::gather() %>%
     tidyr::separate(.data$key,
       into = c("channel", "none"),
